@@ -121,23 +121,22 @@ def get_collection_queryables(api: API, request: Union[APIRequest, Any],
             HTTPStatus.NOT_FOUND, headers, request.format, 'NotFound', msg)
 
     LOGGER.debug('Creating collection queryables')
-    try:
-        LOGGER.debug('Loading feature provider')
-        p = load_plugin('provider', get_provider_by_type(
-            api.config['resources'][dataset]['providers'], 'feature'))
-    except ProviderTypeError:
+
+    p = None
+    for pt in ['feature', 'coverage', 'record']:
         try:
-            LOGGER.debug('Loading coverage provider')
+            LOGGER.debug(f'Loading {pt} provider')
             p = load_plugin('provider', get_provider_by_type(
-                api.config['resources'][dataset]['providers'], 'coverage'))  # noqa
+                api.config['resources'][dataset]['providers'], pt))
+            break
         except ProviderTypeError:
-            LOGGER.debug('Loading record provider')
-            p = load_plugin('provider', get_provider_by_type(
-                api.config['resources'][dataset]['providers'], 'record'))
-    except ProviderGenericError as err:
+            LOGGER.debug(f'Providing type {pt} not found')
+
+    if p is None:
+        msg = 'queryables not available for this collection'
         return api.get_exception(
-            err.http_status_code, headers, request.format,
-            err.ogc_exception_code, err.message)
+            HTTPStatus.BAD_REQUEST, headers, request.format,
+            'NoApplicableError', msg)
 
     queryables = {
         'type': 'object',
@@ -178,10 +177,13 @@ def get_collection_queryables(api: API, request: Union[APIRequest, Any],
                 queryables['properties'][k]['x-ogc-role'] = 'primary-instant'  # noqa
 
     if request.format == F_HTML:  # render
+        api.set_dataset_templates(dataset)
+
         queryables['title'] = l10n.translate(
             api.config['resources'][dataset]['title'], request.locale)
 
         queryables['collections_path'] = api.get_collections_url()
+        queryables['dataset_path'] = f'{api.get_collections_url()}/{dataset}'
 
         content = render_j2_template(api.tpl_config,
                                      'collections/queryables.html',
@@ -380,8 +382,12 @@ def get_collection_items(
 
     LOGGER.debug('processing property parameters')
     for k, v in request.params.items():
-        if k not in reserved_fieldnames and k in list(p.fields.keys()):
-            LOGGER.debug(f'Adding property filter {k}={v}')
+        if k not in reserved_fieldnames:
+            if k in list(p.fields.keys()):
+                LOGGER.debug(f'Adding property filter {k}={v}')
+            else:
+                LOGGER.debug(f'Adding additional property filter {k}={v}')
+
             properties.append((k, v))
 
     LOGGER.debug('processing sort parameter')
@@ -444,7 +450,8 @@ def get_collection_items(
                 geometry_column_name=provider_def.get('geom_field'),
             )
         except Exception:
-            msg = f'Bad CQL string : {cql_text}'
+            msg = 'Bad CQL text'
+            LOGGER.error(f'{msg}: {cql_text}')
             return api.get_exception(
                 HTTPStatus.BAD_REQUEST, headers, request.format,
                 'InvalidParameterValue', msg)
@@ -502,9 +509,12 @@ def get_collection_items(
             serialized_query_params += '='
             serialized_query_params += urllib.parse.quote(str(v), safe=',')
 
+    if 'links' not in content:
+        content['links'] = []
+
     # TODO: translate titles
     uri = f'{api.get_collections_url()}/{dataset}/items'
-    content['links'] = [{
+    content['links'].extend([{
         'type': 'application/geo+json',
         'rel': request.get_linkrel(F_JSON),
         'title': l10n.translate('This document as GeoJSON', request.locale),
@@ -519,7 +529,7 @@ def get_collection_items(
         'rel': request.get_linkrel(F_HTML),
         'title': l10n.translate('This document as HTML', request.locale),
         'href': f'{uri}?f={F_HTML}{serialized_query_params}'
-    }]
+    }])
 
     if offset > 0:
         prev = max(0, offset - limit)
@@ -531,17 +541,23 @@ def get_collection_items(
                 'href': f'{uri}?offset={prev}{serialized_query_params}'
             })
 
-    if 'numberMatched' in content:
-        if content['numberMatched'] > (limit + offset):
-            next_ = offset + limit
-            next_href = f'{uri}?offset={next_}{serialized_query_params}'
-            content['links'].append(
-                {
-                    'type': 'application/geo+json',
-                    'rel': 'next',
-                    'title': l10n.translate('Items (next)', request.locale),
-                    'href': next_href
-                })
+    next_link = False
+
+    if content.get('numberMatched', -1) > (limit + offset):
+        next_link = True
+    elif len(content['features']) == limit:
+        next_link = True
+
+    if next_link:
+        next_ = offset + limit
+        next_href = f'{uri}?offset={next_}{serialized_query_params}'
+        content['links'].append(
+            {
+                'type': 'application/geo+json',
+                'rel': 'next',
+                'title': l10n.translate('Items (next)', request.locale),
+                'href': next_href
+            })
 
     content['links'].append(
         {
@@ -561,6 +577,7 @@ def get_collection_items(
     l10n.set_response_language(headers, prv_locale, request.locale)
 
     if request.format == F_HTML:  # render
+        api.set_dataset_templates(dataset)
         # For constructing proper URIs to items
 
         content['items_path'] = uri
@@ -836,7 +853,7 @@ def post_collection_items(
     if (request_headers.get(
         'Content-Type') or request_headers.get(
             'content-type')) != 'application/query-cql-json':
-        msg = ('Invalid body content-type')
+        msg = 'Invalid body content-type'
         return api.get_exception(
             HTTPStatus.BAD_REQUEST, headers, request.format,
             'InvalidHeaderValue', msg)
@@ -872,16 +889,18 @@ def post_collection_items(
                 geometry_column_name=provider_def.get('geom_field')
             )
         except Exception:
-            msg = f'Bad CQL string : {data}'
+            msg = 'Bad CQL text'
+            LOGGER.error(f'{msg}: {data}')
             return api.get_exception(
                 HTTPStatus.BAD_REQUEST, headers, request.format,
                 'InvalidParameterValue', msg)
     else:
-        LOGGER.debug('processing Elasticsearch CQL_JSON data')
+        LOGGER.debug('processing CQL_JSON data')
         try:
             filter_ = CQLModel.parse_raw(data)
         except Exception:
-            msg = f'Bad CQL string : {data}'
+            msg = 'Bad CQL text'
+            LOGGER.error(f'{msg}: {data}')
             return api.get_exception(
                 HTTPStatus.BAD_REQUEST, headers, request.format,
                 'InvalidParameterValue', msg)
@@ -1161,6 +1180,7 @@ def get_collection_item(api: API, request: APIRequest,
     l10n.set_response_language(headers, prv_locale, request.locale)
 
     if request.format == F_HTML:  # render
+        api.set_dataset_templates(dataset)
         content['title'] = l10n.translate(collections[dataset]['title'],
                                           request.locale)
         content['id_field'] = p.id_field
