@@ -54,6 +54,7 @@ from pygeoapi.util import (
     JobStatus,
     ProcessExecutionMode,
     RequestedProcessExecutionMode,
+    RequestedResponse,
     Subscriber
 )
 
@@ -107,14 +108,21 @@ class BaseManager:
         else:
             return load_plugin('process', process_conf['processor'])
 
-    def get_jobs(self, status: JobStatus = None) -> list:
+    def get_jobs(self,
+                 status: JobStatus = None,
+                 limit: Optional[int] = None,
+                 offset: Optional[int] = None
+                 ) -> dict:
         """
         Get process jobs, optionally filtered by status
 
         :param status: job status (accepted, running, successful,
                        failed, results) (default is all)
+        :param limit: number of jobs to return
+        :param offset: pagination offset
 
-        :returns: `list` of jobs (identifier, status, process identifier)
+        :returns: dict of list of jobs (identifier, status, process identifier)
+                  and numberMatched
         """
 
         raise NotImplementedError()
@@ -187,6 +195,7 @@ class BaseManager:
                                data_dict: dict,
                                requested_outputs: Optional[dict] = None,
                                subscriber: Optional[Subscriber] = None,
+                               requested_response: Optional[RequestedResponse] = RequestedResponse.raw.value  # noqa
                                ) -> Tuple[str, None, JobStatus]:
         """
         This private execution handler executes a process in a background
@@ -197,27 +206,34 @@ class BaseManager:
         :param p: `pygeoapi.process` object
         :param job_id: job identifier
         :param data_dict: `dict` of data parameters
-        :param requested_outputs: `dict` specify the subset of required
-            outputs - defaults to all outputs.
-            The value of any key may be an object and include the property
-            `transmissionMode` - defaults to `value`.
-            Note: 'optional' is for backward compatibility.
+        :param requested_outputs: `dict` optionally specifying the subset of
+                                  required outputs - defaults to all outputs.
+                                  The value of any key may be an object and
+                                  include the property `transmissionMode`
+                                  (defaults to `value`)
+                                  Note: 'optional' is for backward
+                                  compatibility.
         :param subscriber: optional `Subscriber` specifying callback URLs
+        :param requested_response: `RequestedResponse` optionally specifying
+                                   raw or document (default is `raw`)
 
         :returns: tuple of None (i.e. initial response payload)
                   and JobStatus.accepted (i.e. initial job status)
         """
-        _process = dummy.Process(
-            target=self._execute_handler_sync,
-            args=(p, job_id, data_dict, requested_outputs, subscriber)
-        )
+
+        args = (p, job_id, data_dict, requested_outputs, subscriber,
+                requested_response)
+
+        _process = dummy.Process(target=self._execute_handler_sync, args=args)
         _process.start()
+
         return 'application/json', None, JobStatus.accepted
 
     def _execute_handler_sync(self, p: BaseProcessor, job_id: str,
                               data_dict: dict,
                               requested_outputs: Optional[dict] = None,
                               subscriber: Optional[Subscriber] = None,
+                              requested_response: Optional[RequestedResponse] = RequestedResponse.raw.value  # noqa
                               ) -> Tuple[str, Any, JobStatus]:
         """
         Synchronous execution handler
@@ -229,15 +245,27 @@ class BaseManager:
         :param p: `pygeoapi.process` object
         :param job_id: job identifier
         :param data_dict: `dict` of data parameters
-        :param requested_outputs: `dict` specify the subset of required
-            outputs - defaults to all outputs.
-            The value of any key may be an object and include the property
-            `transmissionMode` - defaults to `value`.
-            Note: 'optional' is for backward compatibility.
+        :param requested_outputs: `dict` optionally specifying the subset of
+                                  required outputs - defaults to all outputs.
+                                  The value of any key may be an object and
+                                  include the property `transmissionMode`
+                                  (defaults to `value`)
+                                  Note: 'optional' is for backward
+                                  compatibility.
         :param subscriber: optional `Subscriber` specifying callback URLs
+        :param requested_response: `RequestedResponse` optionally specifying
+                                   raw or document (default is `raw`)
 
         :returns: tuple of MIME type, response payload and status
         """
+
+        extra_execute_parameters = {}
+
+        # only pass requested_outputs if supported,
+        # otherwise this breaks existing processes
+        if p.supports_outputs:
+            extra_execute_parameters['outputs'] = requested_outputs
+
         self._send_in_progress_notification(subscriber)
 
         try:
@@ -248,13 +276,12 @@ class BaseManager:
                 job_filename = None
 
             current_status = JobStatus.running
-            jfmt, outputs = p.execute(
-                data_dict,
-                # only pass requested_outputs if supported,
-                # otherwise this breaks existing processes
-                **({'outputs': requested_outputs}
-                   if p.supports_outputs else {})
-            )
+            jfmt, outputs = p.execute(data_dict, **extra_execute_parameters)
+
+            if requested_response == RequestedResponse.document.value:
+                outputs = {
+                    'outputs': [outputs]
+                }
 
             self.update_job(job_id, {
                 'status': current_status.value,
@@ -330,7 +357,8 @@ class BaseManager:
             data_dict: dict,
             execution_mode: Optional[RequestedProcessExecutionMode] = None,
             requested_outputs: Optional[dict] = None,
-            subscriber: Optional[Subscriber] = None
+            subscriber: Optional[Subscriber] = None,
+            requested_response: Optional[RequestedResponse] = RequestedResponse.raw.value  # noqa
     ) -> Tuple[str, Any, JobStatus, Optional[Dict[str, str]]]:
         """
         Default process execution handler
@@ -339,12 +367,17 @@ class BaseManager:
         :param data_dict: `dict` of data parameters
         :param execution_mode: `str` optionally specifying sync or async
                                processing.
-        :param requested_outputs: `dict` optionally specify the subset of
-            required outputs - defaults to all outputs.
-            The value of any key may be an object and include the property
-            `transmissionMode` - defaults to `value`.
-            Note: 'optional' is for backward compatibility.
+        :param requested_outputs: `dict` optionally specifying the subset of
+                                  required outputs - defaults to all outputs.
+                                  The value of any key may be an object and
+                                  include the property `transmissionMode`
+                                  (default is `value`)
+                                  Note: 'optional' is for backward
+                                  compatibility.
         :param subscriber: `Subscriber` optionally specifying callback urls
+        :param requested_response: `RequestedResponse` optionally specifying
+                                   raw or document (default is `raw`)
+
 
         :raises UnknownProcessError: if the input process_id does not
                                      correspond to a known process
@@ -356,6 +389,9 @@ class BaseManager:
         job_id = str(uuid.uuid1())
         processor = self.get_processor(process_id)
         processor.set_job_id(job_id)
+        extra_execute_handler_parameters = {
+            'requested_response': requested_response
+        }
 
         if execution_mode == RequestedProcessExecutionMode.respond_async:
             job_control_options = processor.metadata.get(
@@ -406,6 +442,11 @@ class BaseManager:
         }
         self.add_job(job_metadata)
 
+        # only pass subscriber if supported, otherwise this breaks
+        # existing managers
+        if self.supports_subscribing:
+            extra_execute_handler_parameters['subscriber'] = subscriber
+
         # TODO: handler's response could also be allowed to include more HTTP
         # headers
         mime_type, outputs, status = handler(
@@ -413,10 +454,7 @@ class BaseManager:
             job_id,
             data_dict,
             requested_outputs,
-            # only pass subscriber if supported, otherwise this breaks existing
-            # managers
-            **({'subscriber': subscriber} if self.supports_subscribing else {})
-        )
+            **extra_execute_handler_parameters)
 
         return job_id, mime_type, outputs, status, response_headers
 
