@@ -30,8 +30,10 @@
 # =================================================================
 
 import logging
+from http import HTTPStatus
 
-from pygeoapi.provider.base import ProviderGenericError
+from pygeoapi.provider.base import (
+    ProviderGenericError, ProviderItemNotFoundError)
 
 LOGGER = logging.getLogger(__name__)
 
@@ -53,6 +55,7 @@ class BaseTileProvider:
         self.format_type = provider_def['format']['name']
         self.mimetype = provider_def['format']['mimetype']
         self.options = provider_def.get('options')
+        self.tile_type = None
         self.fields = {}
 
     def get_layer(self):
@@ -124,14 +127,16 @@ class BaseTileProvider:
 
 class ProviderTileQueryError(ProviderGenericError):
     """provider tile query error"""
-    pass
+    default_msg = 'Tile not found'
 
 
-class ProviderTileNotFoundError(ProviderGenericError):
+class ProviderTileNotFoundError(ProviderItemNotFoundError):
     """provider tile not found error"""
-    pass
+    default_msg = 'Tile not found (check logs)'
 
 
 class ProviderTilesetIdNotFoundError(ProviderTileQueryError):
     """provider tileset matrix query error"""
-    pass
+    default_msg = 'Tileset id not found'
+    http_status_code = HTTPStatus.NOT_FOUND
+    ogc_exception_code = 'NotFound'
